% Calculate the material stiffness matrix.
%   Call as:
%     D = tangent_moduli(m, context)
%   where
%     m=material
%    context=structure with optional field:
%                kind = string with possible values lambda,
%                       lambda_shear, bulk, bulk_shear
%
%
% the output arguments are
%     D=matrix 3X3
%
function D = tangent_moduli(self, context)
    if isfield(context,'kind')
        switch context.kind
            case 'lambda'
                D = tangent_moduli_lambda(self, context);
            case 'lambda_shear'
                D = tangent_moduli_shear(self, context);
            case 'bulk'
                D = tangent_moduli_bulk(self, context);
            case 'bulk_shear'
                D = tangent_moduli_bulk_shear(self, context);
        end
    else
        E = get_E(self, context);
        nu = get_nu(self, context);
        switch self.type
            case 'strain'
                D = E / (1 + nu) / (1 - 2*nu) * [
                    (1-nu), nu, 0;
                    nu, (1-nu), 0;
                    0 0 (1-2*nu)/2];
            otherwise
                D = E / (1 - nu^2) * [
                    1, nu, 0;
                    nu, 1, 0;
                    0 0 (1-nu)/2];
        end
    end
    return;
end

function D = tangent_moduli_lambda(self, context)
    E = get_E(self, context);
    nu = get_nu(self, context);
    lambda = E * nu / (1 + nu) / (1 - 2*(nu));
    switch self.type
        case 'strain'
            m1 = [1 1 0]';
            D = lambda * m1 * m1';
        otherwise
            error (' Not available');
    end
    return;
end

function D = tangent_moduli_shear(self, context)
    E = get_E(self, context);
    nu = get_nu(self, context);
    mu     = E / (2 * (1 + nu));
    switch self.type
        case 'strain'
            mI = diag([1 1 0.5]);
            D = 2 * mu * mI;
        otherwise
            error (' Not available');
    end
    return;
end

function D = tangent_moduli_bulk(self, context)
    E = get_E(self, context);
    nu = get_nu(self, context);
    B = E / 3 / (1 - 2*(nu));
    switch self.type
        case 'strain'
            m1 = [1 1 0]';
            D = B * m1 * m1';
        otherwise
            error (' Not available');
    end
    return;
end

function D = tangent_moduli_bulk_shear(self, context)
    E = get_E(self, context);
    nu = get_nu(self, context);
    mu     = E / (2 * (1 + nu));
    switch self.type
        case 'strain'
            D = 2 * mu * [2/3*[2 -1; -1 2] zeros(2,1); zeros(1,2) 1 ];
        otherwise
            error (' Not available');
    end
    return;
end
