% Update material state.  Return the updated material state, and the
% requested quantity (default is the stress).
%   Call as:
%     [out,newms] = update(m, ms, context)
%  where
%     m=material
%     ms = material state
%     context=structure
%        with mandatory fields
%           F1= deformation gradient
%        and optional fields
%           output=type of quantity to output, and interpreted by the
%           particular material; [] is returned when the material does not
%           recognize the requested quantity to indicate uninitialized
%           value.  It can be tested with isempty ().
%              output ='Cauchy' - Cauchy stress; this is the default
%                      when output type is not specified.
%              output ='2ndPK' - 2nd Piola-Kirchhoff stress;
%           It is assumed that stress is output in 3-component vector form
%           for plane stress, and four-component vector form for plane strain.
%   The output arguments are
%     out=requested quantity
%     newms=new material state; don't forget that if the update is final
%           the material state newms must be assigned and stored.  Otherwise
%           the material update is lost!
%
function [out, newms] = update (self, ms, context)
    gradU = context.gradU;
    E=1/2*(gradU + gradU');
    Ev = strain_tensor_to_vector (self,E);
    D  = tangent_moduli (self, context);
    ETv = thermal_strains(self,context.dT);
    stress = D * (Ev-ETv);
    switch self.type
        case 'strain' % output through-the-thickness stress
            pEv=(Ev-ETv);
            stress = [stress; D(1,2)*pEv(1)+D(2,1)*pEv(2)];
    end
    if isfield(context,'output')
        switch context.output
            case 'Cauchy'
                out = stress;
            case 'vol_strain'
                out = sum(Ev(1:2));
            otherwise
                out = [];
        end
    else
        out = stress;
    end
    newms = ms;
    return;
end